@@ -6,7 +6,8 @@ Advanced Interface Switcher
 Abstract interface object
 '''
 
-import gtk, hildon, dbus
+from __future__ import with_statement
+import gtk, hildon, dbus, threading
 from advpowcommon.ui.Banners import HildonBanner
 from advpowcommon.ui.Animation import DITPBCachedAnimator
 from advpowcommon.ui.ToggleControl import ToggleControlMaemo5
@@ -20,22 +21,24 @@ class InterfaceState():
 
 class AbstractInterface():
 
+  __interface_check = 'ps aux | grep -m 1 \'%s\' | grep -v \'grep\' | awk {\'print $3\'}'
+
   def __init__(self, bus=None, cb_status_area=None, 
                cb_show_settings_dialog=None):
-    self.__name = 'IFace'
-    self.__processing = False
+    self.__running = False
+    self.__processing = threading.Lock()
     self.__bus = bus
     self.__set_status_area_icon = cb_status_area
     self.__show_settings_dialog = cb_show_settings_dialog
     self.__interfaces = {}
     self.__lcl = locale.get()
     self.interfaces = {}
-    self.indication = 'on'
+    self.network = None
     self.state = {}
     self.__config = config.get()
     self.force_startup_mode = False
     self.pre_init()
-    self.__name = self.state['name']
+    self.__name = self.state.pop('name') if 'name' in self.state else 'IFace'
     self.__config_path = 'settings/%s' % (self.__name.lower())
     self.__button = hildon.Button(gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, 
                                   hildon.BUTTON_ARRANGEMENT_VERTICAL)
@@ -72,6 +75,7 @@ class AbstractInterface():
     if start_state or self.force_startup_mode:
       self.__switch_mode(self.get_modes().index(self.get_config_value('mode-on-startup')), start_state == InterfaceState.off)
 
+    self.__running = True
     #print self.__name, self.state['value'], self.state['mode'], self.state['strength']
     #self.__show_switch_dialog()
     #self.__show_switch_dialog()
@@ -100,6 +104,9 @@ class AbstractInterface():
   def get_modes(self):
     return list(self.state['modes'])
 
+  def get_network_name(self):
+    return self.network
+
   def set_config_option(self, key, values):
     self.__config.set_option('%s/%s' % (self.__config_path, key), values)
 
@@ -113,9 +120,11 @@ class AbstractInterface():
     self.__destroy()
 
   def __destroy(self, button=None):
-    self.__destroy_signals()
-    self.__destroy_interfaces()
-    #print '%s (%s): destroyed' % (self.__name, self.__class__)
+    if hasattr(self, 'state'):
+      self.__destroy_signals()
+      self.__destroy_interfaces()
+      #print '%s (%s): destroyed' % (self.__name, self.__class__)
+    self.__running = False
 
   def __init_signals(self):
     for key in self.state['dbus']:
@@ -153,65 +162,72 @@ class AbstractInterface():
                                         path=self.state['dbus'][x]['path']
                                        )
       receiver_match.remove()
+      delattr(self, 'received_%s' % x)
       #print '%s: destroyed' % (x)
       return True
     return False
 
+  def __set_value(self, value):
+    update = value != None and value != self.state['value']
+    if update:
+      self.state['value'] = value
+      if self.state_control:
+        self.state_control.set_active(value)
+    return update
+
+  def __set_mode(self, value):
+    update = value != None and value != self.state['mode']
+    if update:
+      self.state['mode'] = value
+      if self.mode_control:
+        self.mode_control.set_active(value)
+    return update
+
+  def __set_strength(self, value):
+    update = value != None and value != self.state['strength']
+    if update:
+      self.state['strength'] = value
+    return update
+
   def __received_signal(self, key, *args, **kwargs):
     #try: 
     value, mode, strength = getattr(self, 'process_signal_%s' % key)(*args, **kwargs)
     #except Exception, e: print 'Error: ', e
-    show_banner = False
-    update_ui = False
-    if value != None and value != self.state['value']:
-      self.state['value'] = value
-      show_banner = True
-      update_ui = True
-      if self.state_control:
-        self.state_control.set_active(value)
-    if mode != None and mode != self.state['mode']:
-      self.state['mode'] = mode
-      show_banner = True
-      if self.mode_control:
-        self.mode_control.set_active(mode)
-    if strength != None and strength != self.state['strength']:
-      self.state['strength'] = strength
-      update_ui = True
-    if update_ui:
+
+    updated_value = self.__set_value(value)
+    updated_mode = self.__set_mode(mode)
+    updated_strength = self.__set_strength(strength)
+
+    if updated_value or updated_strength:
       self.__update_ui()
-    windex = self.state['value']
-    if self.get_config_value('show-popups') and show_banner and self.state['banner'][windex]:
-      text = '%s: %s' % \
-             (self.__name, self.__lcl.get(self.state['text'][windex]))
-      if len(self.state['modes']) > 1:
-        text = '%s (%s)' % \
-               (text, self.__lcl.get_item(self.state['modes'][self.state['mode']]))
-      HildonBanner.show_information(self.__button, text)
-    #print self.__name, value, mode, strength
+    if updated_value or updated_mode:
+      self.__show_banner()
+    #print self.__name, value, mode, strength, updated_value, updated_mode, updated_strength
 
   def __init_interfaces(self):
     for x in self.interfaces:
+      #print self.__name, x, 
       self.__init_interface(x)
 
   def __init_interface(self, x, force=False):
     try:
-      loaded = True
-      if self.interfaces[x][3] and not force:
-          loaded = get_shell_command_output('ps aux | grep \'%s\' | grep -v \'grep\' | awk {\'print $3\'}' % 
-                                            (self.interfaces[x][3])) != ''
-      if loaded:
-        self.__interfaces[x] = dbus.Interface(self\
-                                              .get_bus()\
-                                              .get_object(self.interfaces[x][0], 
-                                                          self.interfaces[x][1], 
-                                                          introspect=self.interfaces[x][4], 
-                                                          follow_name_owner_changes=True), 
-                                              self.interfaces[x][2])
-      return True
+      if x in self.interfaces and all(self.interfaces[x][:3]):
+        loaded = get_shell_command_output(self.__interface_check % 
+                                            (self.interfaces[x][3])) != '' \
+                   if self.interfaces[x][3] and not force \
+                   else True
+        if loaded:
+          kwargs = {'introspect': self.interfaces[x][4],
+                    'follow_name_owner_changes': True}
+          self.__interfaces[x] = dbus.Interface(self.get_bus()\
+                                                  .get_object(*self.interfaces[x][:2],**kwargs),
+                                                self.interfaces[x][2])
+        return loaded
     except Exception, e:
-      del self.__interfaces[x]
+      if x in self.__interfaces:
+        self.__interfaces.pop(x)
       print e
-      return False
+    return False
 
   def reinit_interface(self, x, force=False):
     self.__destroy_interface(x)
@@ -231,7 +247,8 @@ class AbstractInterface():
 
   def __destroy_interface(self, x):
     try:
-      del self.__interfaces[x]
+      if x in self.__interfaces:
+        self.__interfaces.pop(x)
       return True
     except:
       return False
@@ -241,19 +258,27 @@ class AbstractInterface():
     self.__update_ui()
 
   def switched(self):
-    self.__processing = False
     self.stop_animation()
+    #try:
+    #  self.__processing.release()
+    #except:
+    #  pass
 
   def update(self):
     self.__update_ui()
 
   def __update_ui(self):
     value = self.state['value']
-    image = self.state['images'][value]
-    if value == InterfaceState.connected and self.state['strength'] > 0:
-      image = self.state['images_st'][self.state['strength']]
+    strength = self.state['strength']
+    image = self.state['images_st'][strength] \
+      if value == InterfaceState.connected and strength > 0 \
+      else self.state['images'][value]
     self.__set_si(image)
-    self.__button.set_value(self.__lcl.get(self.state['text'][value]))
+    self.__button.set_value( \
+        self.network \
+        if value == InterfaceState.connected and self.get_config_value('show-network-in-sm') and self.network \
+        else self.__lcl.get(self.state['text'][value])
+      )
 
   def __button_clicked(self, button=None, user_data=None):
     return self.__switch(self.get_config_value('show-state-dialog'))
@@ -262,14 +287,12 @@ class AbstractInterface():
     return self.__switch(not(self.get_config_value('show-state-dialog')))
 
   def __switch(self, show_dialog):
-    if not self.__processing:
-      self.__processing = True
-      self.start_animation()
-      if show_dialog:
-        gobject.idle_add(self.__show_switch_dialog)
-      else:
-        self.__switch_state(int(self.state['value'] == InterfaceState.off))
-      self.__button.get_toplevel().hide()
+    self.start_animation()
+    if show_dialog:
+      gobject.idle_add(self.__show_switch_dialog)
+    else:
+      self.__switch_state(int(self.state['value'] == InterfaceState.off))
+    self.__button.get_toplevel().hide()
     return True
 
   def __show_switch_dialog(self):
@@ -293,7 +316,7 @@ class AbstractInterface():
 
     #settings_dialog.vbox.set_size_request(-1, min(350, height))
     settings_dialog.show_all()
-    self.__processing = False
+    #self.__processing.release()
     response = settings_dialog.run()
     if response == gtk.RESPONSE_DELETE_EVENT:
       #settings_dialog.destroy()
@@ -330,12 +353,8 @@ class AbstractInterface():
       chstate = min(1, int(cstate))
       value = self.__get_state_control_value()
       if chstate != value:
-        try:
           self.__switch_state(chstate)
-        except:
-          self.switched()
-          self.__update_ui()
-      elif cstate != value: 
+      elif cstate != value:
         control.set_active(value)
 
   def __turn_mode(self, widget=None, control=None, cmode=None):
@@ -345,39 +364,43 @@ class AbstractInterface():
       chmode = max(0, control.get_active())
       #if self.state['value'] != InterfaceState.off and chmode != self.state['mode']:
       if chmode != self.state['mode']:
-        try:
           self.__switch_mode(chmode)
-        except:
-          self.switched()
-          self.__update_ui()
-      elif cmode != self.state['mode']: 
+      elif cmode != self.state['mode']:
         control.set_active(self.state['mode'])
 
   def __switch_state(self, new_state):
-    cstate = min(1, self.state['value'])
-    new_state = max(0, min(new_state, len(self.state['text']) - 1))
-    if new_state != cstate:
-      self.__button.set_value('...')
-      try:
-        if new_state:
-          self.turn_on()
-        else:
-          self.turn_off()
-      except:
-        self.switched()
-        self.__update_ui()
-    else:
-      self.__processing = False
+    with self.__processing:
+    #if self.__processing.acquire():
+      cstate = min(1, self.state['value'])
+      new_state = max(0, min(new_state, len(self.state['text']) - 1))
+      if new_state != cstate:
+        self.__button.set_value('...')
+        try:
+          if new_state:
+            self.turn_on()
+          else:
+            self.turn_off()
+        except:
+          self.__fallback_switched()
+      #else:
+      #  self.__processing.release()
 
   def __switch_mode(self, new_mode, startup=False):
-    cmode = max(0, self.state['mode'])
-    new_mode = max(0, min(new_mode, len(self.state['modes']) - 1))
-    if new_mode != cmode:
-      try:
-        self.turn_mode(new_mode, startup)
-      except:
-        self.switched()
-        self.__update_ui()
+    with self.__processing:
+    #if self.__processing.acquire():
+      cmode = max(0, self.state['mode'])
+      new_mode = max(0, min(new_mode, len(self.state['modes']) - 1))
+      if new_mode != cmode:
+        try:
+          self.turn_mode(new_mode, startup)
+        except:
+          self.__fallback_switched()
+      #else:
+      #  self.__processing.release()
+
+  def __fallback_switched(self):
+    self.switched()
+    self.__update_ui()
 
   def turn_mode(self, mode, startup):
     pass
@@ -397,8 +420,20 @@ class AbstractInterface():
 
   def __set_sai(self, key):
     if any(self.state['show']):
-      #print self.__name, self.state['value'], self.state['show'][self.state['value']], self.get_config_value('show-sa-icon')
       if self.state['show'][self.state['value']] and self.get_config_value('show-sa-icon'):
         self.__set_status_area_icon(self.__name, key)
       else:
         self.__set_status_area_icon(self.__name, None, False)
+
+  def __show_banner(self):
+    windex = self.state['value']
+    if self.get_config_value('show-popups') and self.state['banner'][windex]:
+      text = '%s: %s' % \
+             (self.__name, 
+             self.network \
+               if windex == InterfaceState.connected and self.network
+               else self.__lcl.get(self.state['text'][windex]))
+      if len(self.state['modes']) > 1:
+        text = '%s (%s)' % \
+               (text, self.__lcl.get_item(self.state['modes'][self.state['mode']]))
+      HildonBanner.show_information(self.__button, text)
