@@ -7,58 +7,66 @@ GSM interface object
 '''
 
 import dbus, gobject
-from advpowcommon.util.execn import *
 from advifswlib.interfaces.Abstract import InterfaceState, AbstractInterface
 
 class GSMInterface(AbstractInterface):
 
-  phone_iface = 'com.nokia.phone.net'
-  phone_path = '/com/nokia/phone/net'
-  phone_iiface = 'Phone.Net'
+#  radio_access_signal = 'radio_access_technology_change'
 
-  cellular_system_signal = 'cellular_system_state_change'
-  radio_access_signal = 'radio_access_technology_change'
-  strength_signal = 'signal_strength_change'
-  operator_signal = 'operator_name_change'
+  ssc_status = [('offline', 'inactive',
+                 'simless', 'tablet_state',
+                 'blocked_sim', 'sim_locked'),
+                ('activation_failed', 'selection_failed'),
+                ('online','sim_ready', 'initialize'),
+                ('activation_pending',)]
 
-  status_on = (3,4,5,9,10,11)
-  status_connected = (0,1,2)
-
-  gsm_module_iface = 'com.nokia.phone.SSC'
-  gsm_module_path = '/com/nokia/phone/SSC'
-  gsm_module_signal = 'modem_state_changed_ind'
-  sim_iface = 'com.nokia.phone.SIM'
-  sim_path = '/com/nokia/phone/SIM'
-  sim_iiface = 'Phone.Sim'
+  #'sim_mode_als2' - what's that...
+  #status_on = (3,4,5,9,10,11)
+  #status_connected = (0,1,2)
 
   def pre_init(self):
+    self.interfaces = {'Phone': ['com.nokia.phone.net',
+                                 '/com/nokia/phone/net',
+                                 'Phone.Net',
+                                 None,
+                                 False],
+                       'SSC': ['com.nokia.phone.SSC',
+                               '/com/nokia/phone/SSC',
+                               'com.nokia.phone.SSC',
+                               None,
+                               True],
+                       'SIM': ['com.nokia.phone.SIM',
+                               '/com/nokia/phone/SIM',
+                               'Phone.Sim',
+                               None,
+                               False]}
     self.state = {'name': 'GSM',
-                  'dbus': {'on': {'name': self.cellular_system_signal, 
-                                  'iface': self.phone_iiface, 
-                                  'path': self.phone_path, 
+                  'dbus': {'on': {'name': 'cellular_system_state_change',
+                                  'iface': self.interfaces['Phone'][2],
+                                  'path': self.interfaces['Phone'][1],
                                   'args': {}},
-                           'connected': {'name': self.gsm_module_signal, 
-                                         'iface': self.gsm_module_iface, 
-                                         'path': self.gsm_module_path, 
+                           'connected': {'name': 'modem_state_changed_ind',
+                                         'iface': self.interfaces['SSC'][2],
+                                         'path': self.interfaces['SSC'][1],
                                          'args': {}},
-                           'strength': {'name': self.strength_signal, 
-                                        'iface': self.phone_iiface, 
-                                        'path': self.phone_path, 
+                           'strength': {'name': 'signal_strength_change',
+                                        'iface': self.interfaces['Phone'][2],
+                                        'path': self.interfaces['Phone'][1],
                                         'args': {}},
-                           'operator': {'name': self.operator_signal, 
-                                        'iface': self.phone_iiface, 
-                                        'path': self.phone_path, 
+                           'operator': {'name': 'operator_name_change',
+                                        'iface': self.interfaces['Phone'][2],
+                                        'path': self.interfaces['Phone'][1],
                                         'args': {}},
                           },
-                  'images': ('ifsw_statusarea_cell_off', 
-                             'ifsw_statusarea_cell_signal_0', 
+                  'images': ('ifsw_statusarea_cell_off',
+                             'ifsw_statusarea_cell_signal_0',
                              'ifsw_statusarea_cell_signal_5',
                              'ifsw_statusarea_cell_connecting'),
-                  'images_st': ('ifsw_statusarea_cell_signal_0', 
-                                'ifsw_statusarea_cell_signal_1', 
-                                'ifsw_statusarea_cell_signal_2', 
-                                'ifsw_statusarea_cell_signal_3', 
-                                'ifsw_statusarea_cell_signal_4', 
+                  'images_st': ('ifsw_statusarea_cell_signal_0',
+                                'ifsw_statusarea_cell_signal_1',
+                                'ifsw_statusarea_cell_signal_2',
+                                'ifsw_statusarea_cell_signal_3',
+                                'ifsw_statusarea_cell_signal_4',
                                 'ifsw_statusarea_cell_signal_5'),
                   'text': ['Off', 'On', '<>', 'Connecting'],
                   'show': (True, True, False, True),
@@ -67,12 +75,7 @@ class GSMInterface(AbstractInterface):
                   'strength': 0,
                   'modes': ('gsm-3g+gsm', 'gsm-2g', 'gsm-3g'),
                   'mode': 0}
-    self.interfaces = {'Phone': [self.phone_iface, self.phone_path, 
-                                 self.phone_iiface, '', False], 
-                       'SSC': [self.gsm_module_iface, self.gsm_module_path, 
-                               self.gsm_module_iface, '', True],
-                       'SIM': [self.sim_iface, self.sim_path, 
-                               self.sim_iiface, '', False]}
+
 
   def process_signal_on(self, state, arg1, arg2):
     return self.get_status()
@@ -82,39 +85,38 @@ class GSMInterface(AbstractInterface):
       value = str(value)
       result = None
       strength = None
-      if value == 'inactive':
+      if value in self.ssc_status[InterfaceState.off]:
         result = InterfaceState.off
-      elif value == 'activation_pending':
-        #result = InterfaceState.on
+      elif value in self.ssc_status[InterfaceState.connecting]:
         result = InterfaceState.connecting
-      elif value == 'online':
+      elif value in self.ssc_status[InterfaceState.connected]:
         result = InterfaceState.connected
         if self.get_config_value('show-signal-strength'):
           strength = self.get_strength()
         sp_info = self.get_interface('SIM').get_service_provider_name()
         if sp_info:
           self.process_signal_operator(None, sp_info[0])
-      else: #'activation_failed', 'selection_failed'
-        print value
+      else:
+        print 'unknown:', value
         result = InterfaceState.on
       #return 1 + ['activation_pending', 'online'].index(str(value)), 
       return result, self.get_mode(), strength
-    except:
+    except Exception, e:
+      print self.get_name(), e
       return None, None, None
 
   def process_signal_strength(self, arg1, arg2):
-    if self.get_config_value('show-signal-strength'):
-      return None, None, self.get_strength_value(arg1)
-    else:
-      return None, None, None
+      return None, \
+             None, \
+             self.get_strength_value(arg1) \
+             if self.get_config_value('show-signal-strength') \
+             else None
 
   def process_signal_operator(self, arg1, opname, empty=None, mnc=None, mcc=None):
     opname = str(opname)
-    #if not opname:
-    #  self.state['text'][InterfaceState.connected] = '<>'
-    #else:
     if opname:
-      self.state['text'][InterfaceState.connected] = opname
+      #self.state['text'][InterfaceState.connected] = opname
+      self.network = opname
     return None, None, None
 
   def get_status(self):
