@@ -6,15 +6,45 @@ Advanced Interface Switcher
 WLAN interface object
 '''
 
-import dbus, gobject
+from __future__ import with_statement
+import dbus, gobject, threading
 from advifswlib.interfaces.Abstract import InterfaceState, AbstractInterface
 from advpowcommon.util.execn import *
 
 class WLANInterface(AbstractInterface):
 
-  wifi_module_path = '/org/freedesktop/Hal/Manager'
-  wifi_module_iface = 'org.freedesktop.Hal.Manager'
-  wifi_module_signal = None
+  #connection changed
+  #0 DBUS_TYPE_STRING              service type or empty string
+  #1 DBUS_TYPE_UINT32              service attributes
+  #2 DBUS_TYPE_STRING              service id or empty string
+  #3 DBUS_TYPE_STRING              network type or empty string
+  #4 DBUS_TYPE_UINT32              network attributes
+  #5 DBUS_TYPE_ARRAY (BYTE)        network id or empty string (ESSID)
+  #6 DBUS_TYPE_STRING              error that occured; empty string on success
+  #7 DBUS_TYPE_UINT32              state of the network connection
+
+  #network search
+  #0 DBUS_TYPE_STRING              the network type for which search is done
+  #1 DBUS_TYPE_UINT32              state of the network connection
+
+  #4 network attributes
+  #ICD_NW_ATTR_IAPNAME		0x01000000
+  #ICD_NW_ATTR_SILENT		0x02000000
+  #ICD_NW_ATTR_AUTOCONNECT	0x04000000
+  #ICD_NW_ATTR_SRV_PROVIDER	0x10000000
+  #ICD_NW_ATTR_LOCALMASK	0x00FFFFFF
+
+  #7 state of the network connection
+  #ICD_STATE_DISCONNECTED		0
+  #ICD_STATE_CONNECTING			1
+  #ICD_STATE_CONNECTED			2
+  #ICD_STATE_DISCONNECTING		3
+  #ICD_STATE_LIMITED_CONN_ENABLED	4
+  #ICD_STATE_LIMITED_CONN_DISABLED	5
+  #ICD_STATE_SEARCH_START		8
+  #ICD_STATE_SEARCH_STOP		9
+  #ICD_STATE_INTERNAL_ADDRESS_ACQUIRED	15
+
   wifi_cmd = {'up': 'sudo /usr/bin/advifsw/wlan_up.sh',
               'down': 'sudo /usr/bin/advifsw/wlan_down.sh',
               'unload': 'sudo /usr/bin/advifsw/wlan_unload.sh',
@@ -26,31 +56,47 @@ class WLANInterface(AbstractInterface):
   # off: down > unload
   wifi_turn_off_cmd = [wifi_cmd['down'], wifi_cmd['unload']]
   wifi_drivers = ['wl12xx', 'wl1251']
-  
-  wifi_icd2_path = '/com/nokia/icd2'
-  wifi_icd2_iface = 'com.nokia.icd2'
-  wifi_icd2_state_signal = 'state_sig'
-  wifi_icd2_statistics_signal = 'statistics_sig'
-  wifi_icdui_path = '/com/nokia/icd_ui'
-  wifi_icdui_iface = 'com.nokia.icd_ui'
 
   def pre_init(self):
+    self.__lock = threading.Lock()
     self.__strength_timeout = None
     self.__autooff = None
     self.force_startup_mode = True
-    self.has_be = get_shell_command_output(self.wifi_cmd['check_be']) != ''
+    self.__has_be = get_shell_command_output(self.wifi_cmd['check_be']) != ''
+
+    self.interfaces = {'Manager': [None,
+                                   '/org/freedesktop/Hal/Manager',
+                                   'org.freedesktop.Hal.Manager',
+                                   None,
+                                   False],
+                       'ICD2': ['com.nokia.icd2',
+                               '/com/nokia/icd2',
+                               'com.nokia.icd2',
+                               '/usr/sbin/icd2',
+                               False],
+                       'ICD_UI': ['com.nokia.icd_ui',
+                                  '/com/nokia/icd_ui',
+                                  'com.nokia.icd_ui',
+                                  '/usr/sbin/icd2',
+                                  False],
+                       'WLANCOND.req': ['com.nokia.wlancond',
+                                        '/com/nokia/wlancond/request',
+                                        'com.nokia.wlancond.request',
+                                        None,
+                                        False]}
+
     self.state = {'name': 'WLAN',
-                  'dbus': {'on': {'name': self.wifi_module_signal, 
-                                  'iface': self.wifi_module_iface, 
-                                  'path': self.wifi_module_path, 
+                  'dbus': {'on': {'name': None,
+                                  'iface': self.interfaces['Manager'][2],
+                                  'path': self.interfaces['Manager'][1],
                                   'args': {'member_keyword': 'member'}},
-                           'connected': {'name': self.wifi_icd2_state_signal, 
-                                         'iface': self.wifi_icd2_iface, 
-                                         'path': self.wifi_icd2_path, 
-                                         'args': {}},
-                           'statistics': {'name': self.wifi_icd2_statistics_signal, 
-                                          'iface': self.wifi_icd2_iface, 
-                                          'path': self.wifi_icd2_path, 
+                           'state': {'name': 'state_sig',
+                                     'iface': self.interfaces['ICD2'][2],
+                                     'path': self.interfaces['ICD2'][1],
+                                     'args': {}},
+                           'statistics': {'name': 'statistics_sig',
+                                          'iface': self.interfaces['ICD2'][2],
+                                          'path': self.interfaces['ICD2'][1],
                                           'args': {}}},
                   'images': ('ifsw_statusarea_wlan_off', 
                              'ifsw_statusarea_wlan_signal_0', 
@@ -70,13 +116,8 @@ class WLANInterface(AbstractInterface):
                   'strength': 0,
                   'modes': ['wlan-stock'],
                   'mode': 0}
-    if self.has_be:
-      self.state['modes'] = ['wlan-stock', 'wlan-bleeding-edge']
-    self.interfaces = {'ICD': [self.wifi_icd2_iface, self.wifi_icd2_path, 
-                               self.wifi_icd2_iface, '/usr/sbin/icd2', False],
-                       'ICD_UI': [self.wifi_icdui_iface, self.wifi_icdui_path, 
-                                  self.wifi_icdui_iface, '/usr/sbin/icd2', 
-                                  False]}
+    if self.__has_be:
+      self.state['modes'].append('wlan-bleeding-edge')
 
   def post_init(self):
     self.set_config_option('disconnected-timeout', (-1,10,30,60,300))
@@ -98,7 +139,7 @@ class WLANInterface(AbstractInterface):
         self.__cancel_get_strength()
     return result, mode, None
 
-  def process_signal_connected(self, *args):
+  def process_signal_state(self, *args):
     result = None
     if len(args) == 8 and args[3].startswith('WLAN_'):
       if args[7] == 2:
@@ -106,9 +147,9 @@ class WLANInterface(AbstractInterface):
         self.__cancel_autooff()
         if self.get_config_value('show-signal-strength'):
           self.__strength_timeout = gobject.timeout_add(1000, self.get_strength)
-        ap_bytename = args[5]
-        apname = ''.join([str(char) for char in ap_bytename if char])
-      elif args[7] == 0 and self.state['value'] != 0: #State: On
+        ap_bytename = self.get_interface('WLANCOND.req').status()[0]
+        self.network =  ''.join([str(char) for char in ap_bytename])
+      elif args[7] == 0 and self.state['value'] != InterfaceState.off: #State: On
         result = InterfaceState.on
         if self.state['value'] == InterfaceState.disconnecting:
           #schedule autoff if needed
@@ -125,6 +166,7 @@ class WLANInterface(AbstractInterface):
         self.__cancel_get_strength()
       else:
         result = self.state['value']
+    #print 'args:', args
     return result, None, None
 
   def process_signal_statistics(self, *args):
@@ -137,34 +179,34 @@ class WLANInterface(AbstractInterface):
       return InterfaceState.off, mode, None
     else:
       try:
-        self.get_interface('ICD').state_req()
+        self.get_interface('ICD2').state_req()
       except:
-        self.reinit_interface('ICD')
+        self.reinit_interface('ICD2')
       return InterfaceState.on, mode, 0
 
   def turn_on(self):
     if self.state['value'] == InterfaceState.off:
-      #run_async(self.run_commands, (self.wifi_turn_on_cmd,))
-      gobject.idle_add(self.run_commands, [self.wifi_cmd['load%s' % self.state['mode']], self.wifi_cmd['up']])
+      #gobject.idle_add(self.run_commands, self.wifi_cmd['load%s' % self.state['mode']], self.wifi_cmd['up'])
+      run_async(self.run_commands, [self.wifi_cmd['load%s' % self.state['mode']], self.wifi_cmd['up']])
     elif self.state['value'] == InterfaceState.connected:
       try:
-        self.get_interface('ICD').disconnect_req(dbus.UInt32(0x8000))
+        self.get_interface('ICD2').disconnect_req(dbus.UInt32(0x8000))
       except:
-        self.reinit_interface('ICD')
+        self.reinit_interface('ICD2')
     return False
 
   def turn_off(self):
     self.__cancel_autooff()
     if self.state['value'] != InterfaceState.off:
       self.disconnect()
-      #run_async(self.run_commands, (self.wifi_turn_off_cmd,))
-      gobject.idle_add(self.run_commands, self.wifi_turn_off_cmd)
+      run_async(self.run_commands, self.wifi_turn_off_cmd)
+      #gobject.idle_add(self.run_commands, *self.wifi_turn_off_cmd)
     return False
 
-  def run_commands(self, cmds):
-    run_shell_command(' && '.join(cmds))
-#    self.scan_request()
-    self.switched()
+  def run_commands(self, *cmds):
+    with self.__lock:
+      run_shell_command(' && '.join(cmds))
+      self.switched()
 
   def show_connection_dialog(self):
     try:
@@ -175,13 +217,13 @@ class WLANInterface(AbstractInterface):
 
   def scan_request(self):
     try:
-      self.get_interface('ICD').scan_req(dbus.UInt32(0), dbus.Array([], signature='s'))
+      self.get_interface('ICD2').scan_req(dbus.UInt32(0), dbus.Array([], signature='s'))
     except:
-      self.reinit_interface('ICD')
+      self.reinit_interface('ICD2')
 
   def get_strength(self):
     if self.state['value'] == InterfaceState.connected and self.get_config_value('show-signal-strength'):
-      self.get_interface('ICD').statistics_req()
+      self.get_interface('ICD2').statistics_req()
       self.__strength_timeout = gobject.timeout_add(5000, self.get_strength)
     return False
 
@@ -199,7 +241,8 @@ class WLANInterface(AbstractInterface):
       else:
         self.disconnect()
         # down > unload > load%mode% > up
-        gobject.idle_add(self.run_commands, [self.wifi_cmd['down'], self.wifi_cmd['load%s' % mode], self.wifi_cmd['up']])
+        #gobject.idle_add(self.run_commands, self.wifi_cmd['down'], self.wifi_cmd['load%s' % mode], self.wifi_cmd['up'])
+        run_async(self.run_commands, [self.wifi_cmd['down'], self.wifi_cmd['load%s' % mode], self.wifi_cmd['up']])
     self.switched()
 
   def __cancel_autooff(self):
@@ -212,7 +255,7 @@ class WLANInterface(AbstractInterface):
 
   def __get_mode(self, driver):
     config = 0
-    if self.has_be:
+    if self.__has_be:
       if driver == '':
         try:
           config = self.state['modes'].index(self.get_config_value('mode-on-startup'))
@@ -225,6 +268,6 @@ class WLANInterface(AbstractInterface):
   def disconnect(self):
     if self.state['value'] == InterfaceState.connected:
       try:
-        self.get_interface('ICD').disconnect_req(dbus.UInt32(0x8000))
+        self.get_interface('ICD2').disconnect_req(dbus.UInt32(0x8000))
       except:
-        self.reinit_interface('ICD')
+        self.reinit_interface('ICD2')
